from django.db import models

from anfema_django_utils.db.triggers import DenyFieldUpdateTrigger


class RelatedModel(models.Model):
    content = models.TextField(default="default content")


class DenyFieldUpdateTriggerTestModel(models.Model):
    set_once_flag = models.BooleanField()
    related_model = models.ForeignKey(RelatedModel, on_delete=models.PROTECT)
    content = models.TextField(default="default content")

    class Meta:
        constraints = (
            DenyFieldUpdateTrigger('set_once_flag', name='test_deny_set_once_flag_update'),
            DenyFieldUpdateTrigger('related_model_id', name='test_deny_related_model_id_update'),
        )



from django.db import models
from django.db.models.constants import LOOKUP_SEP
from django.utils.functional import cached_property


"""Collection of database functions (mostly postgres specific)"""


class Epoch(models.Func):
    """
    Postgres function to extract the epoch (i.e. seconds) from an interval.
    """

    function = 'EXTRACT'
    template = "%(function)s('epoch' from %(expressions)s)"


class ExtractPathText(models.Func):
    """
    Postgres specific function to extract text from a JSONField at the specified path.

    Usage:
        ``ExtractPathText('jsonfield__path__to__key')``
    """

    function = 'jsonb_extract_path_text'

    def __init__(self, json_field_path: str, **extra):
        if LOOKUP_SEP not in json_field_path:
            raise ValueError("Expected lookup path in the form <jsonfield>__<path__to__key>")
        json_field, *path_elements = json_field_path.split(LOOKUP_SEP)
        extra.setdefault('output_field', self.output_field)
        super().__init__(models.F(json_field), *map(models.Value, path_elements), **extra)

    @cached_property
    def output_field(self):
        return models.TextField()


class NormalizeUmlauts(models.Func):
    """
    Function to replace german umlauts in strings ('ä' -> 'ae'...).
    """

    arity = 1

    def __int__(self, expression, output_field=None):
        output_field = output_field or models.TextField()
        super().__init__(expression, output_field)

    @property
    def template(self):
        replacements = [('ä', 'ae'), ('ö', 'oe'), ('ü', 'ue'), ('Ä', 'Ae'), ('Ö', 'Oe'), ('Ü', 'Ue'), ('ß', 'ss')]
        template = 'REPLACE(' * len(replacements) + '%(expressions)s'
        for umlaut, replacement in replacements:
            template += f", '{umlaut}', '{replacement}')"
        return template


class RegexpReplace(models.Func):
    """
    Postgres function to substitute text using regular expressions.
    """

    function = 'REGEXP_REPLACE'

    def __init__(self, expression, pattern, replacement=models.Value(''), flags=models.Value(''), **extra):
        super().__init__(expression, pattern, replacement, flags, **extra)


class SubquerySum(models.Subquery):
    """
    Useful for subquery-sum aggregations in query annotations:
    Requires a subquery, column name and output_field type.
    """

    template = '(SELECT sum(_sum."%(column)s") FROM (%(subquery)s) _sum)'

    def __init__(self, queryset, column, output_field=None, **extra):
        super().__init__(queryset, output_field, column=column, **extra)



# Generated by Django 4.1.3 on 2022-11-23 09:08

import anfema_django_utils.db.triggers
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='RelatedModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.TextField(default='default content')),
            ],
        ),
        migrations.CreateModel(
            name='DenyFieldUpdateTriggerTestModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('set_once_flag', models.BooleanField()),
                ('content', models.TextField(default='default content')),
                (
                    'related_model',
                    models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='test_app.relatedmodel'),
                ),
            ],
        ),
        migrations.AddConstraint(
            model_name='denyfieldupdatetriggertestmodel',
            constraint=anfema_django_utils.db.triggers.DenyFieldUpdateTrigger(
                field='set_once_flag', name='test_deny_set_once_flag_update'
            ),
        ),
        migrations.AddConstraint(
            model_name='denyfieldupdatetriggertestmodel',
            constraint=anfema_django_utils.db.triggers.DenyFieldUpdateTrigger(
                field='related_model_id', name='test_deny_related_model_id_update'
            ),
        ),
    ]



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import importlib
import os
import sys


sys.path.insert(0, os.path.abspath('..'))  # noqa

from anfema_django_utils import __version__  # noqa


# -- Project information -----------------------------------------------------
project = 'anfema-django-utils'
copyright = '2022, anfema GmbH'
author = 'anfema GmbH'
version = __version__
release = __version__

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx_autodoc_typehints',
    "sphinx.ext.intersphinx",
    'sphinx_copybutton',
    "sphinx_rtd_theme",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages. See the documentation for
# a list of builtin themes.
html_theme = 'sphinx_rtd_theme'
html_css_files = [
    "extend-rtd-style.css",
]
add_module_names = False
autodoc_preserve_defaults = True
autodoc_docstring_signature = True
autosummary_generate = True  # Turn on sphinx.ext.autosummary

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]


# -- Intersphinx
# https://www.sphinx-doc.org/en/master/usage/extensions/intersphinx.html
intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "django": ("https://docs.djangoproject.com/en/stable/", "https://docs.djangoproject.com/en/stable/_objects/"),
}

# For objects which cannot be resolved by intersphinx
qualname_overrides = {
    "django.contrib.auth.base_user.BaseUserManager": "django.contrib.auth.models.BaseUserManager",
    "django.contrib.auth.base_user.AbstractBaseUser": "django.contrib.auth.models.AbstractBaseUser",
    "builtins.object": "object",
}


def get_canonical(obj: type) -> str:
    modules = obj.__module__.split(".")
    if canonical_fullname := qualname_overrides.get(f"{obj.__module__}.{obj.__qualname__}", None):
        return canonical_fullname
    for i in range(n_modules := len(modules)):
        if not (canonical_name := ".".join(modules[: n_modules - i - 1])):
            break
        canonical_module = importlib.import_module(canonical_name)
        if getattr(canonical_module, obj.__qualname__, None) is obj:
            return ".".join([canonical_name, obj.__qualname__])
    return f"{obj.__module__}.{obj.__qualname__}"


def adapt_base_classes(app, name, obj, options, bases) -> None:
    for idx, base in enumerate(bases):
        main_package, *_ = base.__module__.split(".")
        if main_package == "lys":
            continue  # Don`t try to adapt lys classes

        # If the base is not a lys project class display the full class name
        bases[idx] = f":py:class:`{get_canonical(base)}`"


def setup(app):
    app.connect("autodoc-process-bases", adapt_base_classes)



"""This module provides caching related utilities.

.. note:: This module is only supported with Python >= 3.9

.. versionadded:: 1.5
"""
from __future__ import annotations


__all__ = ("cached_call",)

import sys  # isort: skip

if not (sys.version_info.major == 3 and sys.version_info.minor >= 9):
    raise ImportError('The caching module is only available with Python >= 3.9')

import contextlib
import functools
import inspect
from collections import OrderedDict, abc
from typing import TYPE_CHECKING

from super_hash import super_hash


if TYPE_CHECKING:
    from types import ClassMethodDescriptorType
    from typing import Any, Callable


class _CachedCall:
    __slots__ = ("__wrapped__", "__signature__", "__cache__", "__dict__", "_maxsize")

    def __new__(cls, func: Callable | ClassMethodDescriptorType, *, maxsize: int = None) -> functools.partial:
        if inspect.ismethod(func):
            raise TypeError("Callable must not be an instant method.")
        elif inspect.isclass(func):
            raise TypeError("Callable must not be a class.")
        return functools.wraps(func)(super().__new__(cls))

    def __init__(self, func: functools.partial, maxsize: int) -> None:
        self.__wrapped__ = func
        self.__signature__ = inspect.signature(self)
        self.__cache__ = OrderedDict()

        # Ensure the cache`s size is at least 1
        self._maxsize = 1 if maxsize is not None and maxsize < 1 else maxsize

    def __get__(self, instance, owner=None) -> Callable:
        if instance is None:
            return self

        @functools.wraps(self.__wrapped__)
        def cached_call_proxy(cls, *args, **kwargs):
            return self(cls, *args, **kwargs)

        @functools.wraps(self.clear_call)
        def clear_call_proxy(cls, *args, **kwargs):
            return self.clear_call(cls, *args, **kwargs)

        cached_call_proxy.clear = self.clear
        cached_call_proxy.clear_call = clear_call_proxy.__get__(instance, owner)

        return cached_call_proxy.__get__(instance, owner)

    def __call__(self, *args, **kwargs) -> Any:
        hash_key = self._hash_parameters(*args, **kwargs)
        if hash_key in self.__cache__:
            return self.__cache__[hash_key]
        if len(self.__cache__) == self._maxsize:
            self.__cache__.pop(next(iter(self.__cache__)))  # Remove the first cache item if maxsize has been reached

        return self.__cache__.setdefault(hash_key, self._call_wrapped(*args, **kwargs))

    def __str__(self) -> str:
        return str(self.__wrapped__)

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} object at 0x{id(self):x}: {repr(self.__wrapped__)}>"

    def _call_wrapped(self, *args, **kwargs) -> Any:
        """Calls the wrapped function directly without caching the call."""
        return getattr(self.__wrapped__, "__func__", self.__wrapped__)(*args, **kwargs)

    def _hash_parameters(self, *args, **kwargs) -> str:
        """Calculates a hash value for the given parameters"""
        bound_parameters = self.__signature__.bind(*args, **kwargs)
        bound_parameters.apply_defaults()
        return super_hash(bound_parameters.arguments)

    def clear_call(self, *args, **kwargs) -> None:
        """Clearing the callable`s cache for a specific call."""
        with contextlib.suppress(KeyError):
            self.__cache__.pop(self._hash_parameters(*args, **kwargs))

    def clear(self) -> None:
        """Clearing the callable`s complete cache."""
        self.__cache__.clear()


@functools.singledispatch
def cached_call(maxsize: int = 128):
    """Decorator to wrap a function with a memoizing callable similar to the :func:`functools.lru_cache` decorator.

    You can use the decorator on lambdas, functions, methods, classmethods and staticmethods.

    .. note:: Some callables may not be cachable in certain implementations of Python. For example, in CPython,
          built-in functions defined in C provide no metadata about their arguments.

    Unlike :func:`functools.lru_cache`, :func:`cached_call` provides the possibility to invalidate the cache for a
    specific call. The cache keeps references to the arguments and return values until they age out of the cache
    or until the cache is cleared via :meth:`clear_call` or :meth:`clear`.

    To cache the result :func:`cached_call` uses the positional and keyword arguments and creates a hash value using the
    `super_hash <https://github.com/jeff-hykin/super_hash>`_ library. Hence, the positional and keyword arguments must
    be hashable with `super_hash <https://github.com/jeff-hykin/super_hash>`_.

    If a method is cached, the ``self`` instance attribute is included in the cache. The same applies for the ``cls``
    attribute of a cached ``classmethod``.

    Distinct argument patterns does not result in separate cache entries, i.e. :code:`f(a=1, b=2)` and
    :code:`f(b=2, a=1)` have the same cache entry. You should consider that arguments with a default value are taken
    also into account for caching. I.e. a function with a signature :code:`f(a=None, b=True)` results in the same cache
    entry for the calls :code:`f(a=None)` and :code:`f(b=True)`. For methods, the calls
    :code:`Foo.bar(foo, *args, **kwargs)` and :code:`foo.bar(*args, **kwargs)` will be considered the same and thus also
    result in the same cache entry.

    If *user_function* is specified, it must be a callable. This allows the :func:`cached_call` decorator to be applied
    directly to a user function, leaving the *maxsize* at its default value of 128:

    .. code-block::

        @cached_call(maxsize=32)
        def foo(*args):
            ...

        @cached_call
        def bar(*args):
            ...

    If *maxsize* is set to ``None``, the cache can grow without bound.

    The original underlying callable is accessible through the :attr:`__wrapped__` attribute.

    To clear a cache entry for a specific call you can use :func:`clear_call` with passing the arguments, for which call
    the cache entry shall be cleared. To reset the complete cache, call :meth:`clear`:

    .. code-block::

        @cached_call
        def foo(*args):
            return uuid.uuid4()

        >>> foo(1)  # returns a new UUID
        UUID('9306c73b-ff0a-439b-901c-c347ac548904')
        >>> foo(1)  # returns the cached value
        UUID('9306c73b-ff0a-439b-901c-c347ac548904')
        >>> foo(2)  # returns a new UUID
        UUID('2e7ca126-7943-46cc-a81b-0fbd30b07969')
        >>> foo.clear_call(1)  # clear the cache only for the call foo(1)

        >>> foo(1)  # returns a new UUID
        UUID('3b3cc8b2-6ec5-4582-8314-926aa625b1fc')
        >>> foo(2)  # returns the cached value
        UUID('2e7ca126-7943-46cc-a81b-0fbd30b07969')
        >>> foo.clear()  # clear the cache for all calls
    """
    if type(maxsize) not in (int, type(None)):
        raise TypeError("cached_call() requires a callable, integer or None.")
    return functools.partial(_CachedCall, maxsize=maxsize)


@cached_call.register
def _(user_function: abc.Callable):
    return _CachedCall(user_function, maxsize=128)



from django.test import TestCase


class ModelfilterTestCase(TestCase):
    modelfilter_css_file = 'modelfilter.css'
    modelfilter_js_file = 'modelfilter.js'

    def test_css_exists(self):
        response = self.client.get('/admin/', follow=True)
        self.assertContains(response, self.modelfilter_css_file)

    def test_js_exists(self):
        response = self.client.get('/admin/', follow=True)
        self.assertContains(response, self.modelfilter_js_file)



from typing import Any, Dict, Iterable, Tuple

from django.db import models
from django.db.models.signals import post_init
from django.dispatch import receiver


LoadedFieldValues = Dict[str, Any]
ChangedFieldValues = Dict[str, Tuple[Any, Any]]


class ChangedFieldValuesMixinBase:
    _loaded_values: LoadedFieldValues

    @property
    def changed_field_values(self) -> ChangedFieldValues:
        changes = {}
        for field_name, old_value in getattr(self, '_loaded_values', {}).items():
            new_value = getattr(self, field_name)
            if old_value != new_value:
                changes[field_name] = (old_value, new_value)
        return changes


class ChangedFieldValuesModelMixin(ChangedFieldValuesMixinBase):
    """Mixin to keep track of changed field values.

    All changes (since loading the model from the database) are available
    in the `changed_field_values` property. The property contains a mapping
    where the key is the changed field name and the value a tuple of the old
    and new field value.

    To just check if a field was changed use the `in` operator:
    >>> 'field_name' in self.changed_field_values
    """

    @classmethod
    def from_db(cls, db, field_names, values) -> models.Model:
        instance = super().from_db(db, field_names, values)
        instance._loaded_values = dict(zip(field_names, values))
        return instance


class SnapshotFieldValuesModelMixin(ChangedFieldValuesMixinBase):
    """Mixin to keep track of changed field values based on snapshots.

    Field value snapshots are created whenever a model instance is initialized.
    This happens on model initialization, creation or `refresh_from_db` calls for example.

    To manually create a field value snapshot, call the :meth:`snapshot_field_values` method.

    Only concrete (and non-deferred) field values are captured currently, for foreign keys
    for example only the `<fk_field>_id` key is present.
    """

    @property
    def current_field_value_snapshot(self) -> LoadedFieldValues:
        return self._loaded_values.copy()

    def snapshot_field_values(self, field_names: Iterable[str] = None) -> None:
        if field_names is None:
            field_names = [f.attname for f in self._meta.concrete_fields if f.attname not in self.get_deferred_fields()]
        self._loaded_values = {field_name: getattr(self, field_name) for field_name in field_names}


@receiver(post_init)
def snapshot_field_values(sender: Any, instance: SnapshotFieldValuesModelMixin, *args, **kwargs):
    if isinstance(instance, SnapshotFieldValuesModelMixin):
        instance.snapshot_field_values()



from django import forms

from .models import DenyFieldUpdateTriggerTestModel


class DenyFieldUpdateTriggerModelForm(forms.ModelForm):
    class Meta:
        model = DenyFieldUpdateTriggerTestModel
        fields = '__all__'



from unittest import SkipTest

from django.db import InternalError, connections
from django.test import TestCase

from anfema_django_utils.db.triggers import DenyFieldUpdateTrigger
from example_proj.test_app.forms import DenyFieldUpdateTriggerModelForm
from example_proj.test_app.models import DenyFieldUpdateTriggerTestModel, RelatedModel


class TestDenyFieldUpdateTrigger(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        db_vendor = connections['default'].vendor
        # TODO: make tests compatible with `mysql` (different exception on trigger violation)
        if db_vendor not in ('postgresql',):
            raise SkipTest(f"DenyFieldUpdateTrigger does not support the {db_vendor} db backend")
        super().setUpClass()

    @classmethod
    def setUpTestData(cls) -> None:
        cls.model_with_triggers = DenyFieldUpdateTriggerTestModel.objects.create(
            set_once_flag=True, related_model=RelatedModel.objects.create()
        )
        cls.model_form = DenyFieldUpdateTriggerModelForm()

    def test_save_changed_unrelated_field_succeeds(self):
        self.model_with_triggers.content = "new content"
        self.model_with_triggers.save()

    def test_save_changed_set_once_flag_fails(self):
        self.model_with_triggers.set_once_flag = False
        with self.assertRaisesMessage(InternalError, "Direct update of “set_once_flag” is forbidden."):
            self.model_with_triggers.save()

    def test_update_changed_set_once_flag_fails(self):
        with self.assertRaisesMessage(InternalError, "Direct update of “set_once_flag” is forbidden."):
            DenyFieldUpdateTriggerTestModel.objects.update(set_once_flag=False)

    def test_save_changed_related_model_fails(self):
        self.model_with_triggers.related_model = RelatedModel.objects.create()
        with self.assertRaisesMessage(InternalError, "Direct update of “related_model_id” is forbidden."):
            self.model_with_triggers.save()

    def test_update_changed_related_model_fails(self):
        new_related_model = RelatedModel.objects.create()
        with self.assertRaisesMessage(InternalError, "Direct update of “related_model_id” is forbidden."):
            DenyFieldUpdateTriggerTestModel.objects.update(related_model=new_related_model)

    def test_form_validation(self):
        model_form = DenyFieldUpdateTriggerModelForm(
            {
                'set_once_flag': False,  # invalid change
                'content': 'content',
                'related_model': self.model_with_triggers.related_model_id,
            },
            instance=self.model_with_triggers,
        )
        error_msg = DenyFieldUpdateTrigger.default_violation_error_message % {'name': 'test_deny_set_once_flag_update'}
        self.assertFormError(model_form, field=None, errors=[error_msg])



import contextlib
import warnings

import django
from django.core.exceptions import ValidationError
from django.db import DEFAULT_DB_ALIAS, NotSupportedError
from django.db.models.constraints import BaseConstraint
from django.utils.translation import gettext_lazy as _


if django.VERSION < (4, 1):
    warnings.warn(f"{__name__}.DenyFieldUpdateTrigger requires django v4.1+")


class DenyFieldUpdateTrigger(BaseConstraint):
    """Creates a database trigger to deny any updates to the specified field.

    Add it like any other `Constraint` to `Meta.constraints`.

    Implemented as constraint to get automatic support for (vendor specific)
    sql in migrations and automatic model validation.

    Supports the `postgresql` and `mysql` backends.
    """

    default_trigger_error_message = _("Direct update of “%(field)s” is forbidden.")

    def __init__(self, field, name, violation_error_message=None, trigger_error_message=None):
        super().__init__(name, violation_error_message=violation_error_message)
        if not field:
            raise ValueError(f"{self.__class__.__qualname__}.field must be set.")
        self.field = field
        if trigger_error_message is not None:
            self.trigger_error_message = trigger_error_message
        else:
            self.trigger_error_message = self.default_trigger_error_message

    def _validate_field_name(self, model):
        if self.field not in [f.attname for f in model._meta.concrete_fields]:
            raise ValueError(f"{self.field} is not a concrete field of {model.__name__}.")

    def constraint_sql(self, model, schema_editor):
        self._validate_field_name(model)
        # we can't embed the trigger sql into the "CREATE TABLE" statement
        warnings.warn(
            f"{self.__class__.__qualname__} doesn't support the {schema_editor.connection.vendor} backend. "
            "Skipping creation of triggers."
        )

    def create_sql(self, model, schema_editor) -> str:
        self._validate_field_name(model)
        name = schema_editor.quote_name(self.name)
        field = schema_editor.quote_name(self.field)
        table = schema_editor.quote_name(model._meta.db_table)
        error_message = schema_editor.quote_value(self.trigger_error_message % {"field": self.field})

        if schema_editor.connection.vendor == "postgresql":
            return f"""
                CREATE FUNCTION {name}() RETURNS trigger
                   LANGUAGE plpgsql AS
                $$BEGIN
                   RAISE EXCEPTION {error_message};
                END;$$;

                CREATE TRIGGER {name}
                   BEFORE UPDATE OF {field} ON {table}
                   FOR EACH ROW
                   WHEN (OLD.{field} != NEW.{field})
                   EXECUTE PROCEDURE {name}();
               """
        elif schema_editor.connection.vendor == "mysql":
            return f"""
                CREATE TRIGGER {name}
                    BEFORE UPDATE ON {table}
                    FOR EACH ROW
                    BEGIN
                        IF OLD.{field} <> NEW.{field} THEN
                            SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = {error_message};
                        END IF;
                    END;
                """

        raise NotSupportedError(
            f"{self.__class__.__qualname__} doesn't support the {schema_editor.connection.vendor} backend."
        )

    def remove_sql(self, model, schema_editor) -> str:
        self._validate_field_name(model)
        name = schema_editor.quote_name(self.name)
        table = schema_editor.quote_name(model._meta.db_table)

        if schema_editor.connection.vendor == "postgresql":
            return f"""
                DROP TRIGGER {name} ON {table};
                DROP FUNCTION {name}();
                """
        elif schema_editor.connection.vendor == "mysql":
            return f"""
                DROP TRIGGER {name};
                """

        raise NotSupportedError(
            f"{self.__class__.__qualname__} doesn't support the {schema_editor.connection.vendor} backend."
        )

    def validate(self, model, instance, exclude=None, using=DEFAULT_DB_ALIAS) -> None:
        if instance.pk is None:
            return

        if exclude and self.field in exclude:
            return

        queryset = model._default_manager.using(using)

        with contextlib.suppress(model.DoesNotExist):
            db_instance = queryset.get(pk=instance.pk)
            old_value = getattr(db_instance, self.field)
            new_value = getattr(instance, self.field)
            if new_value != old_value:
                raise ValidationError(self.get_violation_error_message())

    def deconstruct(self):
        path, args, kwargs = super().deconstruct()
        kwargs["field"] = self.field
        if self.trigger_error_message is not None and self.trigger_error_message != self.default_trigger_error_message:
            kwargs["trigger_error_message"] = self.trigger_error_message
        return path, args, kwargs

    def __repr__(self):
        return f"<{self.__class__.__qualname__}: field={repr(self.field)}>"

    def __eq__(self, other):
        if isinstance(other, DenyFieldUpdateTrigger):
            return (
                self.name == other.name
                and self.field == other.field
                and self.violation_error_message == other.violation_error_message
                and self.trigger_error_message == other.trigger_error_message
            )
        return super().__eq__(other)
